; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 20 - 1602 Character LCD
; Link: https://www.circuitbread.com/tutorials/1602-character-lcd-part-20-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
PSA         EQU 0003h
T0CS        EQU 0005h
NOT_GPPU    EQU 0006h

PS0         EQU 0000h
PS1         EQU 0001h
PS2         EQU 0002h

;----- STATUS Bits -----------------------------------------------------
C           EQU 0000h
Z           EQU 0002h

bit_count   EQU 010h    ;counter of the bits
data_byte   EQU 011h    ;data byte to send to the shift register
lcd_data    EQU 012h    ;byte to send to LCD
data_com    EQU 013h    ;if = 0x10, then data, if = 0 then command
temp        EQU 014h    ;Temporary data
i           EQU 015h    ;Delay variable
j           EQU 016h    ;Delay variable
k           EQU 017h    ;Delay variable

dat         EQU GPIO_GP0_POSITION    ;Data input of the shift register
clk         EQU GPIO_GP2_POSITION    ;Clock input of the shift register
latch       EQU GPIO_GP1_POSITION    ;Latch input of the shift register

RS          EQU 4       ;RS signal of the LCD is bit 4
E           EQU 5       ;E signal of the LCD is bit 5

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)


; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW ~(1<<T0CS)    ;enable GPIO2
    OPTION

    MOVLW ~((1 << dat) | (1 << clk ) | (1 << latch))
    TRIS GPIO           ;Set all pins as outputs
    CLRF GPIO           ;Clear GPIO to set all pins to 0
    CLRF data_com       ;Set RS line low to send the command

    ;Delay after power-up
    MOVLW 0x07          ;Perform delay for 2 second
    CALL DELAY
;-----------LCD initialization------------------
    MOVLW 0x33          ;Preparation
    CALL LCD_SEND
    MOVLW 0x02          ;Perform delay for 200ms
    CALL DELAY
    MOVLW 0x32          ;Set 4-bit interface
    CALL LCD_SEND
    MOVLW 0x28          ;Set 2-line display
    CALL LCD_SEND
    MOVLW 0x08          ;Display off
    CALL LCD_SEND
    MOVLW 0x01          ;Clear display
    CALL LCD_SEND
    MOVLW 0x02          ;Perform delay for 200ms
    CALL DELAY
    MOVLW 0x06          ;Cursor increment to the right, no shift
    CALL LCD_SEND
    MOVLW 0x0C          ;Display on, cursor off
    CALL LCD_SEND
    MOVLW 0x02          ;Perform delay for 200ms
    CALL DELAY
;----------End of the initialization part------------------

    MOVLW 0x85          ;Set line 1, position 5
    CALL LCD_SEND
    BSF data_com, RS    ;Set RS line high to send the data
    ;Send the word "HELLO"
    MOVLW 'H'
    CALL LCD_SEND
    MOVLW 'E'
    CALL LCD_SEND
    MOVLW 'L'
    CALL LCD_SEND
    MOVLW 'L'
    CALL LCD_SEND
    MOVLW 'O'
    CALL LCD_SEND

    CLRF data_com       ;Set RS line low to send the command
    MOVLW 0xC5          ;Set line 2, position 5
    CALL LCD_SEND
    BSF data_com, RS    ;Set RS line high to send the data
    ;Send the word "WORLD"
    MOVLW 'W'
    CALL LCD_SEND
    MOVLW 'O'
    CALL LCD_SEND
    MOVLW 'R'
    CALL LCD_SEND
    MOVLW 'L'
    CALL LCD_SEND
    MOVLW 'D'
    CALL LCD_SEND

LOOP:                   ;Main loop of the program

    GOTO LOOP           ;loop forever

SEND_BYTE:              ;Send one byte to the shift register
    CLRF bit_count      ;Clear the 'bit_count' register
SEND_BIT:               ;Send one bit to the shift register
    BCF GPIO, clk       ;Set the 'clk' pin low
    NOP                 ;No operation to implement the instruction
    BCF GPIO, dat       ;Set the 'dat' pin low
    BTFSC data_byte, 7  ;If the MSB of the 'data_byte' is not 0
    BSF GPIO, dat       ;Then set the 'dat' pin high
    RLF data_byte, F    ;Shift 'data_byte' to the left
    BSF GPIO, clk       ;Set the 'clk' pin high
    INCF bit_count, F   ;Increment the 'bit_count'
    BTFSS bit_count, 3  ;If 'bit_count' is less than 8
    GOTO SEND_BIT       ;Then go to 'SEND_BIT' label
    BSF GPIO, latch     ;Set the 'latch' pin high
    NOP                 ;No operation to implement the instruction
    BCF GPIO, latch     ;Set the 'latch' pin low
    RETLW 0             ;Return from the subroutine

LCD_SEND:               ;Send byte to the LCD
    ;Sending the upper nibble
    MOVWF lcd_data      ;Save the W register content in the 'lcd_data'
    MOVWF data_byte     ;Save the W register content in the 'data_byte'
    SWAPF data_byte, F  ;Swap the nibbles of the 'data_byte'
    MOVLW 0x0F          ;Load 0x0F into the W register
    ANDWF data_byte, F  ;And perform AND operation with 'data_byte'
    MOVF data_com, W    ;Copy 'data_com' into the W register
    IORWF data_byte, F  ;and perform OR with 'data_byte'
    MOVF data_byte, W   ;Copy 'data_byte' into the W register
    MOVWF temp          ;And save it into the 'temp' register
    BSF data_byte, E    ;set the 'E' signal
    CALL SEND_BYTE      ;Send the byte to the shift register
    MOVF temp, W        ;Restore the 'temp' value
    MOVWF data_byte     ;And save it into the 'data_byte' register
    BCF data_byte, E    ;Clear the 'E' signal
    CALL SEND_BYTE      ;Send the byte to the shift register
    ;Sending the lower nibble
    MOVF lcd_data, W    ;Copy 'lcd_data' into the 'data_byte'
    MOVWF data_byte
    MOVLW 0x0F          ;Load 0x0F into the W register
    ANDWF data_byte, F  ;And perform AND operation with 'data_byte'
    MOVF data_com, W    ;Copy 'data_com' into the W register
    IORWF data_byte, F  ;and perform OR with 'data_byte'
    MOVF data_byte, W   ;Copy 'data_byte' into the W register
    MOVWF temp          ;And save it into the 'temp' register
    BSF data_byte, E    ;set the 'E' signal
    CALL SEND_BYTE      ;Send the byte to the shift register
    MOVF temp, W        ;Restore the 'temp' value
    MOVWF data_byte     ;And save it into the 'data_byte' register
    BCF data_byte, E    ;Clear the 'E' signal
    CALL SEND_BYTE      ;Send the byte to the shift register
    MOVLW 12           ;Perform 40 us delay to execute the command
    CALL SHORT_DELAY
    RETLW 0

;-------------Short delay subroutine--------------
SHORT_DELAY:
    MOVWF i             ;Copy the value to the register i
SHORT_DELAY_LOOP:       ;Start short delay loop
    DECFSZ i, F         ;Decrement i and check if it is not zero
    GOTO SHORT_DELAY_LOOP;If not, go to the SHORT_DELAY_LOOP label
    RETLW 0             ;Otherwise return from the subroutine

;-------------Long delay subroutine--------------
DELAY:
    MOVWF i             ;Copy the value to the register i
    MOVWF j             ;Copy the value to the register j
    MOVWF k             ;Copy the value to the register k
DELAY_LOOP:             ;Start delay loop
    DECFSZ i, F         ;Decrement i and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    DECFSZ j, F         ;Decrement j and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    DECFSZ k, F         ;Decrement k and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    RETLW 0             ;Otherwise return from the subroutine

    END INIT        ; Program entry point.